import pygame
import math
import os
import sys
import random
from pygame.locals import *

pygame.init()

def mutable(func):
	# TODO Add optional arg which only resets certain caches
	def newfunc(self, *a, **k):
		self._reset_cache()
		return func(self, *a, **k)
	return newfunc

def cached(func):
	def newfunc(self, *a, **k):
		return self._cache_get(func, a, k)
	return newfunc


class CachedMutable(object):
	def __init__(self, use_lru=False):
		self.__cache = {}

		# Implements LRU, in case memory is an issue
		# NOTE: the LRU is actually not a correct LRU...
		# it is more like "least recently GENERATED",
		# not least recently used.
		# TODO add profiling functions so you know how
		# often it is missing / hitting
		self.__cache_lru = []
		self._cache_is_full = False
		self._use_lru = use_lru
	
	def _cache_set_full(self):
		self._cache_is_full = True
	
	def _reset_cache(self):
		self.__cache = {}
		self.__cache_lru = []

	def _check_lru(self, key):
		self.__cache_lru.append(key)
		if self._cache_is_full:
			# Need to evict
			badkey = self.__cache_lru.pop()
			del self.__cache[badkey]

	def _cache_get(self, func, a, k):
		a_bool = a if a else None 
		k_bool = k if k.keys() else None 
		key  = (func, (a_bool, k_bool))
		try:
			key_in = key in self.__cache
		except TypeError, e:
			print "WARNING: Unhashable args."
			# args are unhashable, just return it
			return func(self, *a, **k)
		else:
			if not key_in:
				# actually run it
				self.__cache[key] = func(self, *a, **k)
				# HAVEN'T TESTED LRU:
				if self._use_lru:
					self._check_lru(key)
			# return the cached value
			return self.__cache[key]





def blur_surface(surface, amt):
	"""
	Blur the given surface by the given 'amount'.  Only values 1 and
	greater are valid.  Value 1 = no blur.
	"""
	if amt < 1.0:
		raise ValueError("Arg 'amt' must be greater than 1.0, "+
				"passed in value is %s"%amt)
	scale = 1.0/float(amt)
	surf_size = surface.get_size()
	scale_size = (int(surf_size[0]*scale), int(surf_size[1]*scale))
	surf = pygame.transform.smoothscale(surface, scale_size)
	surf = pygame.transform.smoothscale(surf, surf_size)
	return surf



class PerspectiveSurface(CachedMutable):
	"""
	Takes a surface, and shrinks / sheers it to correct angle, 
	and positions it at X frontmost pixels away from the left.
	PerspectiveSurface(surface, 0.0, 0.0, 0.0) is the same as a normal 

	NOTE: currently, it is optimized for surfaces which do not change
	orientation. For jumping up, etc, this needs to be fixed (maybe).
	"""
	LOW, MEDIUM, HIGH = range(0, 3)
	def __init__(self, surface, quality=LOW):
		CachedMutable.__init__(self, 50)
		self.surface = surface
		self.quality = quality
	
	@staticmethod
	def slice_into_rows(surface):
		pa = pygame.PixelArray(surface)
		rows = []
		for y in xrange(surface.get_height()):
			rows.append(pa[:,y].make_surface())
		return rows
	
	@cached	
	def _srows_at_angle(self, distance, angle):
		# TODO add smoothscale option for awesome
		if angle >= 1.0 or angle <= 0.0:
			raise ValueError("Angle out of bounds.")
		# Distance is measured in pixels-space :)

		# A value of None distance means that we don't 
		# want to warp the distance at all
		if distance is None:
			distance = 0
		
		# NOTE: distance is not correctly computed here. Need to get 
		# out my physics books / think this out, if its dumb
		# XXX This entire class is retarded XXX
		height, width = self.surface.get_size()
		
		new_dimensions = (max(1, width-distance),
					max(1, int((1-angle)*(height-distance)))) 
		scaled = pygame.transform.scale(self.surface, new_dimensions)
		
		rows = PerspectiveSurface.slice_into_rows(scaled)
		
		new_rows = []
		for y, row in zip(xrange(len(rows)), rows):
			# Calculate the pixel difference
			dist = new_dimensions[1] - y
			width_difference = int(dist*angle)
			new_width = max(new_dimensions[0] - width_difference, 1)
			new_row = pygame.transform.scale(row, (new_width, 1))
			
			# Now scoot it over
			offset =  int(width_difference / 2)
			new_row.scroll(dx=offset)

			new_rows.append(new_row)
			
		return new_rows
	
	@staticmethod
	def unslice_rows(rows):
		height = len(rows)
		if not height:
			raise ValueError("Empy rows")
		width = rows[0].get_width()
		new_surface = pygame.Surface((width, height))
		for y, row in zip(xrange(len(rows)), rows):
			new_surface.blit(row, (0, y))

		return new_surface

		
	
	@cached
	def at_position(self, offset, distance, angle):
		"""
		Returns a surface at the given position. Cached for awesomeness.
		"""
		# Get surface rows
		surface_rows = self._srows_at_angle(distance, angle)
		surf = PerspectiveSurface.unslice_rows(surface_rows)
		# Apply offset
		return surf


class Dimen:
	AUTO = 'auto'
	@staticmethod
	def to_prop(scalar, context):
		s = scalar
		if isinstance(s, basestring):
			# Something fancy
			if 'em' in s or 'ex' in s: # em goddamit
				raise NotImplemented("'em' is not implemented yet.")
			elif s.endsith('%'):
				s = float(s.strip('%'))
				if s.replace('.', ' ').isdigit():
					s = s/100.0
				else:
					raise ValueError("Not valid percent.")
			elif s.endswith('px'):
				s = int(s.strip('px'))
			elif s.isdigit():
				s = int(s)
			elif s.replace('.', ' ').isdigit():
				s = float(s)
		
		if isinstance(s, int):
			# Int, which means pixels
			s = s*1.0/scale
		if isinstance(s, float):
			# Must be prop, success!
			return s
		else:
			raise ValueError("Not recognized measurement") 

	@staticmethod
	def to_pixels(prop, context):
		return int(prop * context)
	
	@staticmethod
	def to_pixels2((p_w, p_h), (w, h)):
		return (int(p_w*w), int(p_h*h))

class Constants:
	class Sample: pass
	class PNG: pass
	class Custom(int): pass

class ImageCache(CachedMutable):
	"""
	1. source_image (Keyed by filename!) just this would hugely improve efficiency
	2. transparency  (with transparency at 64 different levels (most expensive operation))
	3. size changed
	4. rotated (fairly destructive operation)
	5. blurred (to different degrees) 
	6. Black and white, etc (cheap operations)
	"""
	def __init__(self, maxsize, resolution, datafolder):
		self.maxsize = maxsize
		self.resolution = resolution
		self.datafolder = datafolder
		CachedMutable.__init__(self)
	
	@cached
	def load(self, filename, colorkey=Constants.PNG):
		"""
		Loads a file with a given colorkey.  Note that the colorkey is
		not remembered, and thus you cannot load a file twice with
		different color keys.
		If the file is a 
		"""
		fullname = os.path.join(self.datafolder, filename)
		try:
			image = pygame.image.load(fullname)
		except pygame.error, message:
			print 'Cannot load image:', fullname
			raise SystemExit, message
		if colorkey is Constants.PNG:
			image = image.convert_alpha()
		else:
			image = image.convert()
		if colorkey not in (None, Constants.PNG):
			if colorkey is Constants.Sample:
				colorkey = image.get_at((0,0))
			# Else, it must be custom, which is
			# just an int.
			image.set_colorkey(colorkey, RLEACCEL)
		return image

	@cached
	def fade(self, filename, alpha):
		"""
		Returns a version of the given file with all pixels reduced to
		a certain alpha, for a fade out effect.
		NOTE: VERY SLOW.
		"""
		# TODO have it degrade gracefully into using
		# surface.set_alpha() on surfaces that support it.
		surface = self.load(filename)
		if alpha == 256:
			return surface
		else:
			surface = surface.copy()
		surface = self.load(filename).copy()
		size = surface.get_size()
		for y in xrange(size[1]):
			for x in xrange(size[0]):
				r,g,b,a = surface.get_at((x,y))
				surface.set_at((x,y),(r,g,b,min(a, int(alpha))))
		return surface
	
	@cached
	def rotate(self, filename, alpha, radians):
		surface = self.fade(filename, alpha)
		degrees = math.degrees(radians)
		if int(degrees) % 360 == 0: 
			# Don't need to rotate at all
			return surface
		else:
			surface = surface.copy()
		surface = pygame.transform.rotate(surface, degrees)
		return surface
	
	@cached
	def scale(self, filename, alpha, radians, dimensions):
		"""
		Applies above effects, and scales to dimensions, where
		dimensions is proportional to screen height and width.
		"""
		# TODO look into rotozoom
		surface = self.rotate(filename, alpha, radians).copy()
		target_size = Dimen.to_pixels2(dimensions, self.resolution)
		surface = pygame.transform.scale(surface, target_size)
		return surface
	
	@cached
	def blur(self, filename, alpha, radians, dimensions, blur_amount):
		"""
		Applies above effects, and blurs. 
		"""
		surface = self.scale(filename, alpha, radians, dimensions)
		if blur_amount <= 1:
			return surface
		else:
			surface = surface.copy() 
		size = surface.get_size()
		target_size = (int(size[0]/blur_amount), int(size[1]/blur_amount))
		# Scale down
		surface = pygame.transform.scale(surface, target_size)
		# Scale up again
		surface = pygame.transform.scale(surface, size)
		return surface
	
	def blur_new(self, filename, alpha, radians, dimensions, blur_amount):
		"""
		Eventually, this will replace the pixelation algorithm above,
		and the above one will be renamed 'pixelate'
		"""
		surface = self.scale(filename, alpha, radians, dimensions)
		if blur_amount <= 1:
			return surface
		else:
			surface = surface.copy() 
		size = surface.get_size()
		overlays = [surface]
		# TODO later have more complicated / precise algoirthm
		blur_levels = range(1, int(blur_amount)+1)
		for x in blur_levels:
			target_size = (int(size[0]/x), int(size[1]/x))
			# Scale down
			overlay = pygame.transform.scale(surface, target_size)
			# Scale up again
			overlay = pygame.transform.scale(surface, size)
			overlays.append(overlay)
		pygame.transform.average_surfaces(overlays, surface)
		return surface
	
	

	
	#def get(self, filepath=None, alpha=None, radians=None, dimensions=None, blur_amount=None):
	def get(self, filepath=None, alpha=None, radians=None, dimensions=None, blur_amount=None):
		lst = [filepath, alpha, radians, dimensions, blur_amount]
		if any(map(lambda x: x is None, lst)):
			raise ValueError("get() must specify all properties.")
		return self.blur(filepath, alpha, radians, dimensions, blur_amount)


_image_cache = None

# TODO generalize the caching system to support any number of  image
# transformations (so you can register new ones), arbitrary ordering and so on.

class CachedImage(object):
	"""
	The imagecached Image class is very powerful. It facilitates a variety
	of pygame transform operations, and some new ones too, which normally
	would be too expensive to compute.  Note that all operations are in place.
	immutable, ie not in place. 
	"""
	def __init__(self, **k):
		self.image_cache = k.get('image_cache', _image_cache)
		if 'filepath' not in k or 'dimensions' not in k:
			raise ValueError("Must specify filepath and dimensions.")
		self.d = {
			'filepath': None,
			'alpha': 255,
			'radians': 0,
			'dimensions': None,
			'blur_amount': 1.0,
		}
		self.d.update(k)
	
	@property
	def surface(self):
		self._surface = self.image_cache.get(**self.d)
		return self._surface
	
	def get_rect(self):
		return self.surface.get_rect()

	def __call__(self):
		return self.surface

	def _change(self, key, value):
		new_props = dict(self.d)
		new_props[key] = value
		if isinstance(self, Sprite):
			return type(self)(new_props['filepath'],\
				new_props['dimensions'], **new_props)
		return type(self)(**new_props)
	
	def swap_image(self, filepath):
		return self._change('filepath', filepath)
	
	def fade(self, alpha):
		return self._change('alpha', alpha)
	
	def rotate(self, rotate):
		return self._change('radians', radians)
	
	def scale(self, dimensions):
		if isinstance(dimensions, float):
			dimensions = (dimensions, dimensions)
		return self._change('dimensions', dimensions)
	
	def blur(self, blur_amount):
		return self._change('blur_amount', blur_amount)

	def precache(self, properties={}, value=None):
		new_props = dict(self.d)
		if isinstance(properties, basestring):
			key = properties
			try:
				i = iter(value)
			except TypeError:
				new_props[key] = value
				self.image_cache.get(**new_props)
			else:
				for val in value:
					new_props[key] = val
					self.image_cache.get(**new_props)
		else:
			new_props.update(properties)
			self.image_cache.get(**new_props)

		

class Sprite(pygame.sprite.Sprite, CachedImage):
	"""
	A handy sprite to extend
	"""
	def __init__(self, filepath, dimensions, **k):
		# init super classes
		CachedImage.__init__(self, filepath=filepath, 
					dimensions=dimensions, **k)
		pygame.sprite.Sprite.__init__(self)
		#self.rect = self.get_rect()
		self.prect = PRect.from_rect(self.get_rect())
	
	@property
	def rect(self):
		# Should never touch the actual rect
		return self.prect.to_rect()
	
	@property
	def image(self):
		return self.surface

	def swap_image_ip(self, filepath):
		self.d['filepath'] = filepath
		self.prect.size = PRect.from_rect(self.get_rect()).size
	
	def fade_ip(self, alpha):
		self.d['alpha'] = alpha
	
	def rotate_ip(self, radians):
		self.d['radians'] = radians
		self.prect.size = PRect.from_rect(self.get_rect()).size
	
	def scale_ip(self, dimensions):
		if isinstance(dimensions, float):
			dimensions = (dimensions, dimensions)
		self.d['dimensions'] = dimensions
		self.prect.size = PRect.from_rect(self.get_rect()).size
	
	def blur_ip(self, blur_amount):
		self.d['blur_amount'] = blur_amount

	def off_screen(self):
		#         top bottom left right
		result = [False, False, False, False]
		if self.rect.bottom > self.imagecache.resolution[0]:
			result[0] = True
		elif self.rect.top > self.imagecache.resolution[0]:
			result[1] = True
		if self.rect.right > self.imagecache.resolution[1]:
			result[2] = True
		elif self.rect.left > self.imagecache.resolution[1]:
			result[3] = True
		return result



class PRect(pygame.Rect):
	"""
	A reimplementation of rects to allow proportional implementatoin

	It cheats by actually using a rect, but the values are between 
	1 and 6400, with 6400 being the proportional to the screen width.
	"""
	DIMEN = 6400
	@staticmethod
	def from_rect(rect):
		pos = rect.topleft
		size = rect.size
		reso = _image_cache.resolution
		# Adjust everything
		pos = (int(pos[0]*6400.0/reso[0]), 
			int(pos[1]*6400.0/reso[0])) 
		size = (int(size[0]*6400.0/reso[0]), 
			int(size[1]*6400.0/reso[0])) 
		return PRect(pos, size)


	def to_rect(self):
		# Access the "real" topleft and size
		pos = self._rect_topleft
		size = self._rect_size
		# Grab the current resolution
		reso = _image_cache.resolution
		# Adjust everything 
		try:
			pos = (int(pos[0]/6400.0*reso[0]), 
				int(pos[1]/6400.0*reso[0])) 
			size = (int(size[0]/6400.0*reso[0]), 
				int(size[1]/6400.0*reso[0])) 
		except ValueError as e: 
			print "Strange ValueError!", e
			return pygame.Rect( (-1,-1), (1,1) )
		return pygame.Rect(pos, size)
	
	def __getattribute__(self, name):
		if name.startswith('_rect_'):
			return object.__getattribute__(self, name[6:])
		attr = object.__getattribute__(self, name)
		if name == 'to_rect':
			return attr
		if hasattr(attr, '__call__'):
			def newfunc(*args, **kwargs):
				new_args = []
				for arg in args:
					if isinstance(arg, float):
						new_args.append(int(arg*6400))
					elif isinstance(arg, tuple):
						subargs = []
						for subarg in arg:
							subargs.append(int(subarg*6400))
						new_args.append(tuple(subargs))
					else:
						raise NotImplementedError("PRect doesn't have that function yet.")
				attr(*new_args, **kwargs)
			return newfunc
		else:
			if isinstance(attr, int):
				value = attr*1.0 / 6400.0
			elif isinstance(attr, tuple):
				new_value = []
				for val in attr:
					new_value.append(val/6400.0)
				attr = tuple(new_value)
			elif name=='__dict__':
				return attr
			else:
				raise NotImplementedError("PRect doesn't do that yet. 02")
			return attr
	
	def __setattr__(self, name, value):
		#print "SETTING", name, "TO", value
		if isinstance(value, float):
			value = int(value * 6400)
		elif isinstance(value, tuple):
			new_value = []
			for val in value:
				new_value.append(val*6400)
			value = tuple(new_value)
		else:
			raise NotImplementedError("PRect doesn't support this.")
		#print "---->     SETTING", name, "TO", value
		pygame.Rect.__setattr__(self, name, value)


def reset(maxsize=3000, resolution=(640, 480), datafolder=''):
	global _image_cache
	_image_cache = ImageCache(maxsize, resolution, datafolder)

def load(filepath, size):
	return CachedImage(filepath=filepath, dimensions=size)


load_image_cache = {}

DATAFOLDER = 'data'

def load_image(name, colorkey=None):
	key = (name, colorkey)
	if key not in load_image_cache:
		fullname = os.path.join(DATAFOLDER, name)
		try:
			image = pygame.image.load(fullname)
		except pygame.error, message:
			print 'Cannot load image:', name
			raise SystemExit, message
		if name.endswith('png'):
			image = image.convert_alpha()
		else:
			image = image.convert()
		if colorkey is not None:
			if colorkey is -1:
				colorkey = image.get_at((0,0))
			image.set_colorkey(colorkey, RLEACCEL)
		load_image_cache[key] = (image, image.get_rect())
	image, rect = load_image_cache[key] 	
	return image, rect.copy()


#########################################################################
#  SoundCache caches sound like ImageCache, however it does far less, simply
# because it doesn't need too.
#########################################################################

class SoundCache(CachedMutable):
	def __init__(self, maxsize, datafolder):
		self.maxsize = maxsize
		self.datafolder = datafolder
		CachedMutable.__init__(self)
	
	@cached
	def load(self, filepath):
		class NoneSound:
			def play(self): pass
		if not pygame.mixer:
			return NoneSound()
		fullname = os.path.join(self.datafolder, filepath)
		try:
			sound = pygame.mixer.Sound(fullname)
		except pygame.error, message:
			print 'cannot load sound:', wav
			raise SystemExit, message
		return sound

_sound_cache = None

def reset_sound(maxsize=3000, datafolder=''):
	global _sound_cache
	_sound_cache = SoundCache(maxsize, datafolder)

def load_sound(filepath):
	return _sound_cache.load(filepath)





