import pygame
import os
import sys
import random
import math
import time
import itertools
from pygame.locals import *
from imagecache  import *


RECT_UNSET = pygame.Rect((-1, -1), (0, 0))

# TODO make this the same accross camplay & camplayui
FATNESS = 20000

class Event(object):
	ACTIVATE_WAVE = 0
	def __init__(self, widget, value, type=ACTIVATE_WAVE):
		self.widget = widget
		self.value = value
		self.type = type


class Reso:
	w = width = 640
	h = height = 480



class CSS(object):
	@staticmethod
	def to_prop((w, h)):
		# TODO: do w and h separately
		if isinstance(w, str):
			# Something fancy
			if 'em' in w or 'ex' in w: # em goddamit
				raise NotImplemented("'em' is not implemented yet.")
			elif w.endsith('%'):
				w, h = w/100.0, h/100.0
			elif w.endswith('px'):
				w, h = int(w.strip('px').strip()), \
						int(h.strip('px').strip())
		
		if isinstance(w, int) or (isinstance(w, float) and w > 1.0):
			# Int, which means pixels
			w, h = CSS.pixels_to_prop((w, h))
		if isinstance(w, float) and w < 1.0:
			# Must be prop
			return w, h
		else:
			raise ValueError("Not recognized measurement") 

	@staticmethod
	def pixels_to_prop((w, h)):
		return (w*1.0 / Reso.w, h*1.0 / Reso.h)
	
	@staticmethod
	def prop_to_pixels((w_p, h_p)):
		return (int(w_p * Reso.w), int(h_p * Reso.h))
	
	rules = []
	class Rule(object):
		def __init__(self, match, css):
			self.match = match
			self.css = css
		
		def does_match(self, element):
			if isinstance(self.match, basestring):
				return hasattr(element, 'css_class') and \
						self.match in element.css_class
			return isinstance(element, self.match)

	@staticmethod
	def add_rule(match, css):
		CSS.rules.append(CSS.Rule(match, css))

	@staticmethod
	def apply_rules(element, original_css):
		css = dict(original_css)
		for rule in CSS.rules:
			if rule.does_match(element):
				css.update(rule.css)
		return css



class Margin(CSS):
	def __init__(self, left, top=None, right=None, bottom=None):
		self.left = left
		self.top = top if top else left
		self.right = right if right else left
		self.bottom = bottom if bottom else left

	def apply(self, rect):
		# Makes the outer_rect
		rect = rect.copy()
		rect.height += self.bottom
		rect.top += self.top
		rect.left += self.left
		rect.width += self.right
		return rect

class Padding(Margin): 
	pass


class Align(CSS, set):
	KNOWN = 'left right center top bottom middle'
	LEFT, RIGHT, CENTER, TOP, BOTTOM, MIDDLE = KNOWN.split()
	OPPOSITES = {
		LEFT: RIGHT, TOP: BOTTOM
	}
	OPPOSITES.update(dict([(v,k) for k,v in OPPOSITES.iteritems()]))
	"""
	def __init__(self, alignments):
		if any([a for a in alignments if a not in self.KNOWN]):
			raise ValueError("Unknown alignment, use ", self.KNOWN)
		self.alignments = set(alignments)
	"""
	def apply(self, rect, outer_rect):
		rect = rect.copy()
		i_have = lambda a: a in self
		if i_have(self.LEFT):
			rect.left = outer_rect.left
		if i_have(self.RIGHT):
			rect.right = outer_rect.right
		if i_have(self.TOP):
			rect.top = outer_rect.top
		if i_have(self.BOTTOM):
			rect.bottom = outer_rect.bottom
		if i_have(self.MIDDLE):
			rect.centerx = outer_rect.centerx
		if i_have(self.CENTER):
			rect.centery = outer_rect.centery
		return rect

	
	def apply_float(self, rect, other_rect, is_vertical):
		# TODO make  work for vertical float, really hard =S
		# FIXME This entire function does not work, I don't think, or
		# the problem is in the stack code.
		rect = self.apply(rect, other_rect)
		if is_vertical:
			if self.TOP in self:
				rect.top = other_rect.bottom
			elif self.BOTTOM in self:
				rect.bottom = other_rect.top
		else:
			if self.LEFT in self:
				rect.left = other_rect.right
			elif self.RIGHT in self:
				rect.right = other_rect.left
		return rect, other_rect 

	def __repr__(self):
		return "Align(%s)"+set.__repr__(self)

	def flip_h(self):
		# Hacky method, prolly better ways
		def replace(s, a, b):
			if a in s:
				s.remove(a)
				s.add(b)
				return True
		s = set(self)
		replace(s, self.LEFT, self.RIGHT) or replace(s, self.RIGHT, self.LEFT)
		return Align(s)
	
	def flip_v(self):
		"""
		Returns a new Align object with the alignment reversed.
		"""
		# Hacky method, prolly better ways
		def replace(s, a, b):
			if a in s:
				s.remove(a)
				s.add(b)
				return True
		s = set(self)
		replace(s, self.TOP, self.BOTTOM) or replace(s, self.BOTTOM, self.TOP)
		return Align(s)

class Widget(object):
	"""
	Widget is a base class for CamPlayUI widgets.
	"""
	DEFAULT_CSS = {
		'margin': Margin(0),
		'padding': Padding(0),
		'align': Align([Align.LEFT, Align.TOP]),
		'height': 'auto',
		'width': 'auto',
		'color': (0, 0, 0),
		'background-color': (255, 255, 255),
		'background-alpha': (255),
		'active-color': (0, 0, 255),
		'font-size': 0.05, # Proportional to 640x480
		'border-width': 0.0,
	}
	def __init__(self, css={}):
		self.rect = RECT_UNSET
		self.events = []
		# NOTE, might need deep copy here...
		self.css = dict(Widget.DEFAULT_CSS) 
		self.css.update(css)
		self.update_css()
		
	def update_css(self):
		self.css = CSS.apply_rules(self, self.css)
	
	def update_all(self): 
		"""
		Call update_all to call update() all subwidgets. Useful for
		container widgets.
		"""
		pass

	def update(self): 
		"""
		Just like with sprites, use update() for animations or such
		things.
		"""
		pass

	def check_motion(self, mask):
		"""
		Call with a motion mask. This will trigger on_movement() with
		the number of pixels of motion that are in the elements hitbox.
		For containers, it will recurse into sub-elements.
		"""
		pass
	
	def check_presence(self, mask):
		"""
		Call with a matter mask. This will trigger on_presence() with
		the number of pixels of matter that are in the elements hitbox.
		For containers, it will recurse into sub-elements.
		"""
		pass
	
	def on_movement(self, mask):
		"""
		Called when motion is detected on this widget.
		"""
		pass
	
	def on_presence(self, mask):
		"""
		Called when presence is detected under this widget.
		"""
		pass

	def guess_size(self):
		"""
		If you had the entire screen to display this widget, what would be a
		comfortable size? Return two proportions. Numbers greater than 1.0 means 
		you want to use more than the screen size.
		"""
		return (0.3, 0.3)

	def get_events(self):
		events = self.events
		self.events = []
		return events

	def activate(self):
		"""
		Simply adds an event for a wave activation.
		"""
		self.events.append(Event(self, self.value))

	def render_background(self, surface, rect):
		if self.css['background-alpha'] < 255:
			bg_surface = pygame.Surface(rect.size)
			bg_surface.fill(self.css['background-color'])
			bg_surface.set_alpha(self.css['background-alpha'])
			surface.blit(bg_surface, rect.topleft)
	
	def render_border(self, surface, rect):
		if self.css['border-width'] > 0:
			pygame.draw.rect
			bg_surface = pygame.Surface(rect.size)
			bg_surface.fill(self.css['border-color'])
			bg_surface.set_alpha(self.css['background-alpha'])
			surface.blit(bg_surface, rect.topleft)
			

	

class Container(list, Widget):
	"""
	A base class for all widgets that contain other widgets, such as layout
	widgets. It extends list, so you can use all the normal list
	operations (append, remove, pop, etc) for great justice.
	"""
	def __init__(self, *a, **k):
		list.__init__(self, a)
		Widget.__init__(self, **k)

	def update_all(self):
		for widget in self:
			if widget is not None:
				widget.update_all()
				widget.update()

	def check_motion(self, mask):
		for item in self:
			if item is not None:
				item.check_motion(mask)

	def get_events(self):
		events = []
		for item in self:
			if item is not None:
				events += item.get_events()
		return events


class TopHalfContainer(Container):
	"""
	A container which puts all its elements in the top half of the screen.
	Useful for putting buttons on the top of the screen.
	"""
	def __init__(self, *a, **k):
		"""
		Initialize with some elements. Example:
		>>> # Proportion is 0.5, and bottom is False
		>>> thc = TopHalfContainer(elem)
		>>> # Proportion is 0.5, and bottom is False
		>>> thc = TopHalfContainer(elem, proportion=0.3, bottom=True)
		"""
		self.proportion = k.get('proportion', 0.5)
		self.bottom = k.get('bottom', False)
		Container.__init__(self, *a, css=k.get('css', {}))
	
	def render(self, surface, rect=None, proportion=None):
		if not rect:
			rect = surface.get_rect()
		if not proportion:
			proportion = self.proportion
		rect = rect.copy()
		rect.height = int(rect.height * proportion)
		for widget in self:
			widget.render(surface, rect)


class AutoTopHalfContainer(TopHalfContainer):
	def render(self, surface, rect=None):
		# Get the biggest heighted element
		prop = max(map(lambda w: w.guess_size()[1], self))
		# Use self.proportion instead as a "max" value
		prop = min(prop, self.proportion)
		# Use parent render.
		TopHalfContainer.render(self, surface, rect, prop)


class _StackContainer(Container): # Use either Horizontal or Vertical
	def _render(self, is_top, surface, my_rect=None):
		if not my_rect:
			my_rect = surface.get_rect()
		last_r = my_rect.copy()
		last_r.size = (0, 0)
		last_r = self.css['align'].apply(last_r, my_rect)
		# Loop through elements
		# TODO the guess_size thing is broken. 
		# We should return the size of the rect we just
		# rendered, for stacking, etc
		for widget in self:
			p = widget.guess_size()[int(is_top)]
			r = last_r.copy()
			if is_top:
				r.width = my_rect.width
				#r.top = last_r.bottom
				r, __ = self.css['align'].apply_float(r, last_r, True)
				__, r.height = CSS.prop_to_pixels((0, p))
			else:
				r.height = my_rect.height
				r, __ = self.css['align'].apply_float(r, last_r, False)
				r.width, __ = CSS.prop_to_pixels((p, 0))
			# Proportional width
			widget.render(surface, r)
			last_r = r

	def _guess_size(self, is_top):
		if self:
			sizes = [w.guess_size() for w in self]
			max_height = max([size[int(is_top)] for size in sizes])
			total_width = sum([size[int(is_top)] for size in sizes])
			return total_width, max_height
		else:
			return (0, 0)




class VerticalStackContainer(_StackContainer):
	def render(self, surface, my_rect=None):
		return self._render(False, surface, my_rect)

	def guess_size(self):
		return self._guess_size(True)

class HorizontalStackContainer(_StackContainer):
	def render(self, surface, my_rect=None):
		return self._render(False, surface, my_rect)
	
	def guess_size(self):
		return self._guess_size(False)


class GridContainer(Container):
	"""
	A grid container with row and column widths specified, in  
	screen proportions.
	"""
	def __init__(self, *a, **k):
		self.rows = k.get('rows', (0.5, 0.5))
		self.columns = k.get('columns', (0.5, 0.5))
		Container.__init__(self, *a)

	def render(self, surface, my_rect=None):
		# First, render background:
		#print self
		if not my_rect:
			my_rect = surface.get_rect()
		self.render_background(surface, my_rect)
		last_rect = my_rect.copy()
		last_rect.size = (0, 0)
		rects = []

		# First, build all the rects
		for height in self.rows:
			#print "ON ROW: ", height
			#print "ON ROW: ", last_rect
			last_rect.top = last_rect.bottom
			last_rect.left = my_rect.left
			#print "ON ROW: ", last_rect
			for width in self.columns:
				rect = last_rect.copy()
				# Compute the size proportional to my_rect
				print my_rect
				rect.size = (width*my_rect.width, 
						height*my_rect.height)
				# Compute the location
				# TODO once I get the align object working,
				# have this just use align also
				rects.append(rect.copy())
				rect.left = rect.right
				last_rect = rect 
				print "LEFT1", rect.left
		rects = rects[:len(self)]
		#print "THESE ARE THE RECTS", rects
		for widget, rect in zip(self, rects):
			if widget is not None: # Skip empty cells :)
				widget.render(surface, rect)



class AutoGridContainer(Container):
	def __init__(self, *a, **k):
		self.columns = k.get('columns', 3)
		Container.__init__(self, *a)
	"""
	Supports an arbitrary number of elements, which
	it will attempt to grid in the most pleasing way
	Order of attempts:
	First three: [1][2][3]
	Four:        [1][2]    /  [3] [4]
	Five:        [1][2][3] /  [4] [5]
	Six:         [1][2][3]  /  [4] [5] [6]
	Seven:       [1][2][3]  /  [4] [5]  / [6] [7] 
	And so on...
	"""
	def render(self, surface, rect_bounds):
		self.rect = rect_bounds.copy()
		def r_render(lst, r):
			rect = r.copy()
			if len(lst) < 1:
				# Nothing to render
				pass
			elif len(lst) < self.columns+1:
				# A single row
				rect.width = int(rect.width/len(lst))
				for widget in lst:
					widget.render(surface, rect.copy())
					rect.left += rect.width 
			elif len(lst) == self.columns+1 and self.columns not in (2, 1):
				# Never orphans a control...
				rect.height = rect.height/2
				r_render(lst[0:self.columns-1], rect)
				rect.top = rect.top + rect.height
				r_render(lst[self.columns-1:self.columns+1], rect)
			else:
				# Calculates how many rows in total
				rows = int(math.ceil(len(lst)/float(self.columns))) 
				# Render the top row
				rect.height = rect.height/rows
				r_render(lst[:self.columns], rect)
				# Render the rest
				rect.top = rect.top + rect.height
				rect.height = r.height - rect.height
				r_render(lst[self.columns:], rect)

		
		r_render(self, rect_bounds)

	def guess_size(self):
		# 3 high is the "most comfortable" size for this widget
		return (1.0, math.ceil(len(self)*1.0 / self.columns) / 3.0)

class CachedImageWidget(Widget):
	def __init__(self, cachedimage, css={}):
		self.cachedimage = cachedimage
		Widget.__init__(self, css)

	def render(self, surface, rect):
		#print "RECT ---", rect
		s = self.css['margin'].apply(rect)
		image = self.cachedimage.scale(PRect.from_rect(rect).size)
		#print s.topleft
		surface.blit(image(), s.topleft)#s.topleft)
	
	def guess_size(self):
		return self.cachedimage.d['dimensions']

class ImageWidget(Widget):
	def __init__(self, surface, css={}):
		if isinstance(surface, str):
			self.image, __ = load_image(surface)
		else:
			self.image = surface.copy()
		self.rect = self.image.get_rect()
		Widget.__init__(self, css)

	def render(self, surface, rect):
		# Have it stretch to the right size :) (after image caching)
		#r = self.rect.copy()
		s = self.css['margin'].apply(rect)
		surface.blit(self.image, s.topleft)
	
	def guess_size(self):
		# FIXME after ImageCaching
		rect = self.image.get_rect()
		s = self.css['margin'].apply(rect)
		return CSS.to_prop(s.size)

# A HUD class. This can be built off of for quick custom HUDs
class PlayerHUD(TopHalfContainer):
	def __init__(self, player):
		self.player = player
		self.score_label = Label('%i' % player.score)
		self.buffs_stack = HorizontalStackContainer()
		self._last_buffs = []
		stack = HorizontalStackContainer(
			self.score_label, self.buffs_stack)
		# FIXME after Align() works
		#if player.team == player.RIGHT:
		#	stack.css['align'] = stack.css['align'].flip_h()
		TopHalfContainer.__init__(self, stack, proportion=0.15)
	def render(self, surface, rect=None):
		self.score_label.caption  = str(self.player.score)
		if self._last_buffs != self.player.buffs:
			self.buffs_stack[:] = [] # clear list
			css = {'margin': Margin(5) }
			for buff in self.player.buffs:
				self.buffs_stack.append(
					ImageWidget(buff.image, css))
			self._last_buffs = list(self.player.buffs)
		# Assuming its the screen...
		TopHalfContainer.render(self, surface, rect)

class InteractiveWidget(Widget, CachedMutable):
	MOTION_SIZE = 400
	def __init__(self):
		CachedMutable.__init__(self)
		Widget.__init__(self)

	@staticmethod
	def better_check_motion(widget, screen, mask):
		# If more than half the screen is moving, we can assume that
		# the person is adjusting the camera
		w,h = screen.get_size()
		if mask.count() > (w*h/4):
			return
		blobs = mask.connected_components(InteractiveWidget.MOTION_SIZE)
		for blob in blobs:
			widget.check_motion(blob)
	
	@cached
	def get_hitbox(self, size):
		if self.rect == RECT_UNSET:
			return None
		surf = pygame.Surface(size)
		surf.fill((255,255,255))
		pygame.draw.rect(surf, (0,0,0), self.rect)
		#return pygame.mask.from_surface(surf, 0)
		# Stupid hack...
		return pygame.mask.from_threshold(surf, (0, 0, 0), (1, 1, 1))
	
	def check_motion(self, mask):
		hitbox = self.get_hitbox(mask.get_size())
		if not hitbox:
			# For cachedmutable
			self._reset_cache()
		else:
			
			hit = hitbox.overlap_area(mask, (0, 0))
			if hit:
				self.on_movement(hit)


class WaveButton(InteractiveWidget):
	#TODO  remove this class, and replace it with one that doesn't have text
	# but is just a normal container, and thus one might put a label inside of 
	# it.
	def __init__(self, caption, value=None, icon=None, margin=30,
			inner_margin=10, cooldown=3.0, warmup=10.0,
			trigger_at=100, cooldown_delay=10):
		self.caption = caption
		self.value = value
		self.margin = margin
		self.inner_margin = inner_margin
		self.warmup = warmup
		self.cooldown = cooldown
		self.cooldown_delay = cooldown_delay
		self.delay_countdown = 0
		self.trigger_at = trigger_at
		self.icon = icon
		self.heat = 0.0
		if icon:
			self.icon_image, self.icon_rect = load_image(icon)
		InteractiveWidget.__init__(self)
	
	@staticmethod
	def font_size(text):
		if len(text) < 10:
			return 36
		if len(text) < 20:
			return 24
		if len(text) < 30:
			return 14
		return 12

	def render(self, surface, rect):
		rect = rect.copy()
		self.rect = rect
		
		bg_color = pygame.Color(*self.css['background-color'])
		fg_color = pygame.Color(*self.css['color'])
		
		rect.inflate_ip(-self.margin, -self.margin)
		bg_surface = pygame.Surface(rect.size)
		bg_surface.fill(bg_color)
		alpha = 127
		if self.heat > 0:
			# Make alpha look 'brighter'
			alpha += (self.heat / self.trigger_at) * 100
			
			# Make a rect square and centered
			arc_rect = bg_surface.get_rect().copy()
			arc_rect.height -= self.inner_margin
			arc_rect.width = arc_rect.height
			arc_rect.center = bg_surface.get_rect().center
			
			# Make a rect square and centered
			arc_color = pygame.Color(*self.css['active-color'])
			angle = (self.heat / self.trigger_at) * 6.28
			arc_width = min(20, arc_rect.height/2)
			#pygame.gfxdraw.arc(bg_surface, arc_color, arc_rect, 
			#			0, angle, arc_width)
			pygame.draw.arc(bg_surface, arc_color, arc_rect, 
						0, angle, arc_width)
			pygame.draw.arc(bg_surface, arc_color, arc_rect, 
						0.01, angle+0.01, arc_width)
		bg_surface.set_alpha(alpha)
		surface.blit(bg_surface, rect.topleft)
		
		blit_text(surface, self.caption, 
			color=fg_color,
			centered=rect.center)

	def update(self):
		if self.delay_countdown > 0:
			self.delay_countdown =- 1
		elif self.heat > 0:
			self.heat -= self.cooldown
		elif self.heat < 0:
			self.heat = 0.0

	def on_movement(self, amount):
		w, h = self.rect.size
		RESISTANCE = 25000
		RESISTANCE = 2
		self.warmup = 0.75
		self.heat += self.warmup #* (1.0 / RESISTANCE)
		#self.heat += self.warmup * (amount*1.0 / (w * h))
		self.delay_countdown = self.cooldown_delay
		if self.heat > self.trigger_at:
			self.heat = 0.0
			self.activate()
	

	def guess_size(self):
		lines = self.caption.split('\n')
		return (len(lines)*0.1, max(map(len, lines))*0.05)


class WrappedLabel(Widget):
	# Eventually ,this lil son of a bitch will take over aallll the labels
	# TODO fix me when resolutoin shit is completely worked out.
	# I suppose ImageCache should lord over UI for handling resolution
	# stuff, since UI elements are going to be cached also.
	MAX_FONTSIZE = 800 # * Reso.width, essentially
	def __init__(self, caption, css={}):
		self.caption = caption
		Widget.__init__(self, css=css)

	def render(self, surface, rect):
		RESO = Reso.w
		actual_font_size = int(RESO * self.css['font-size'])
		font = pygame.font.Font(None, actual_font_size)
		color = self.css['color']
		justification = self.css['align']
		if 'center' in self.css['align']:
			justification = 1
		elif 'right' in self.css['align']:
			justification = 2
		else:
			justification = 0
		subsurface = render_textrect(self.caption, font, rect, 
					color, justification=justification)
		surface.blit(subsurface, rect.topleft)


class PortraitDialog(GridContainer):
	def __init__(self, caption, portrait, animation_delay=None):
		self.caption = caption
		# Assume to be a CachedImage
		self.portrait = portrait
		self.image = CachedImageWidget(self.portrait)
		#print self.portrait
		#print self.image
		self.animation_delay = animation_delay
		self.ticks = 0
		self.label = WrappedLabel('' if animation_delay else caption)
		self.caption = caption
		self.image_width = self.image.guess_size()[0]
		# Just puts at the bottom of the screen...
		GridContainer.__init__(self, self.image, self.label,
			rows=(1.0,), 
			columns=(self.image_width, 1.0-self.image_width))

	def update(self):
		if self.animation_delay:
			index = self.ticks / self.animation_delay
			if index < len(self.caption):
				self.label.caption = self.caption[:index]
			else:
				self.label.caption = self.caption
			self.ticks += 1
		
	def guess_size(self):
		return (1.0, self.image.guess_size()[1])

class Label(Widget):
	def __init__(self, caption, icon=None, margin=30, inner_margin=10, css={}):
		# TODO: have the thingie extend this
		self.caption = caption
		self.margin = margin
		self.inner_margin = inner_margin
		self.icon = icon
		Widget.__init__(self, css={})

	def render(self, surface, rect):
		rect = rect.copy()
		self.rect = rect
		
		bg_color = pygame.Color(*self.css['background-color'])
		fg_color = pygame.Color(*self.css['color'])
		
		rect.inflate_ip(-self.margin, -self.margin)
		bg_surface = pygame.Surface(rect.size)
		bg_surface.fill(bg_color)
		bg_surface.set_alpha(127)
		surface.blit(bg_surface, rect.topleft)
		
		blit_text(surface, self.caption, 
			color=fg_color, center=rect.midtop)

	def guess_size(self):
		lines = self.caption.split('\n')
		return (len(lines)*0.1, max(map(len, lines))*0.05)

class TMP: a,b,c,d = range(4)

class ModalUI(object):
	def __init__(self, camplay, display, clock):
		self.camplay = camplay
		self.display = display
		self.clock = clock
		self._theme = { }

	def theme(self, **k):
		self._theme.update(**k)
	
	def dialog(self, element, buttons=("Okay", "Cancel"), use_spacing=True, cb=None):
		buttons = AutoGridContainer(*map(WaveButton, buttons))
		"""
		Displays a dialog box, with given buttons, and the body of a 
		given Widget. If the widget is a string, it will automatically
		wrap it around a Label.
		"""
		"""
		if use_spacing:
			buttons = AutoTopHalfContainer(buttons, proportion=1.5)
		container = AutoGridContainer(buttons, element, columns=1)
		"""
		if isinstance(element, basestring):
			element = Label(element)
		container = GridContainer(buttons, element, 
				rows=(0.2, 0.8,), columns=(1.0,))
		
		return self.menu([], container, cb=cb)

	def menu(self, lst, override_widget=None, cb=None, cb_before=None):
		"""
		Displays a menu with the buttons given by the list, and pauses
		until the user makes a selection. Then it returns the result.
		"""
		if not override_widget:
			grid = AutoGridContainer(*map(WaveButton, lst))
			container = AutoTopHalfContainer(grid)
		else:
			container = override_widget
		def callback(image, display):
			if cb_before:
				cb_before(image, display)
			motion_mask = self.camplay.get_motion_mask(image, 2)
			container.update_all()
			InteractiveWidget.better_check_motion(container, image, motion_mask)
			container.render(display, display.get_rect())
			for event in container.get_events():
				value = event.value
				if not value:
					value = event.widget.caption.lower().replace(' ', '')
				return value
			if cb:
				cb(image, display)

			#self.camplay.debug_outline_mask(self.display, motion_mask)
			return False
		return self.pause(callback, 'menu_sprites')

	def player_move_out_of_frame(self, player, side='any', cb=None):
		"""
		Pauses until a given player leaves the frame. If a side is 
		specified, then it will only continue when the player has
		left to that side.
		"""
		def callback(image, display):
			# Tick, so we can know where the player is
			self.camplay.tick(False)
			if not player.is_visible:
				if player.offscreen_side==side or side=='any':
					return True
			if cb:
				cb(image, display)
			return False
		return self.pause(callback, 'menu_sprites')
		
	def score_sequence(self, buttons=("Play again!", "Main menu")):
		def elem_for_player(player, top_elem=False):
			extra_elems = [AutoGridContainer(Label('Score'), Label(str(player.score)))]
			if top_elem:
				extra_elems.insert(0, Label(player.name))
			for label, value in player.stats.iteritems():
				extra_elems.append(AutoGridContainer(Label(label), Label(str(value))))
			return AutoGridContainer(*extra_elems, columns=1)
		
		if self.camplay.players[0].team == self.camplay.SINGLE:
			element = AutoTopHalfContainer(
				elem_for_player(self.camplay.players[0]), proportion=1.0)
		else:
			element = AutoGridContainer(
				elem_for_player(self.camplay.players[0], True),
				elem_for_player(self.camplay.players[1], True),
			)

		return self.dialog(element, buttons)
		
	def pause(self, callback, theme_entry='pause_sprites', on_top=True):
		"""
		Pauses, calling the callback with the current image at 60  FPS
		until it returns a true value. Will display the 'pause_sprite' 
		group if specified.
		"""
		result = False
		quickquit = False
		while not result and not quickquit:
			quickquit = self._check_for_quits()
			image = self.camplay._get_camera_surface()
			self.display.blit(image, (0,0))
			if not on_top:
				result = callback(image, self.display)
			if self._theme.get(theme_entry):
				group = self._theme[theme_entry]
				group.update()
				group.draw(self.display)
			else:
				blit_text(self.display, "PAUSED")
			if on_top:
				result = callback(image, self.display)
			pygame.display.flip()
			self.clock.tick(60)
		return result
	
		
	def calibrate_sequence(self):
		# Does all the hard work for you :)
		self.countdown(2, "Now it's time\n to calibrate the\n camera.")
		self.countdown(5, "Exit the frame,\n so that the camera\n cannot see you.\n%s")
		self.countdown(1, "Taking photo...")
		self.camplay.calibrate()
	
	def get_in_frame_sequence(self, playercount):
		"""Requests that the user get into the frame!"""
		phrase = "%s must be in the frame." %  \
			('You' if playercount < 2 else 'Both players')
		def callback(image, display):
			blit_text(display, phrase,
				center=display.get_rect().midtop)
			if playercount == self._in_frame_blob_count():
				# This stops it
				return True
		self.pause(callback)

		# Now we let give them a little break
		self.countdown(3)

	def loading_delay(self, loading_function, min_time=0):
		"""
		Soon, this will integrate with imagecahce for multithreaded
		precaching.
		"""
		starttime = time.time()
		# Execute the crazy function
		loading_function()
		# Now sleep for the remainder
		time.sleep(time.time() - starttime)
	
	def countdown(self, count=5, message="Resuming in %s seconds", cb=None):
		start_time = time.time()
		label = Label('')
		container = TopHalfContainer(AutoGridContainer(label))
		def callback(image, display):
			now = time.time() 
			seconds_left = count - int(now - start_time)
			label.caption = (message%str(seconds_left)) if '%s' in message \
							else message
			done = seconds_left <= 0
			self._check_for_quits()
			container.update_all()
			if cb:
				cb(image, display)
			container.render(display)
			"""
			blit_text(self.display, label.caption,  color=(128, 0, 0),
				center=self.display.get_rect().midtop)
			"""
			return done
		self.pause(callback, on_top=True)
	
	def _check_for_quits(self):
		for event in pygame.event.get():
			if event.type == QUIT:
				self.camplay.stop()
				sys.exit(0)
			elif event.type == KEYDOWN: 
				if event.key == K_ESCAPE:
					self.camplay.stop()
					sys.exit(0)
				if event.key == K_f:
					pygame.display.toggle_fullscreen()
				if event.key == K_q:
					return True
	
	def _in_frame_blob_count(self):
		mask = self.camplay.gen_matter_mask()
		return len(mask.connected_components(FATNESS)) 
		

	
def blit_text(surface, caption, 
			color=(10,10,10), margin=10, size=48,
			# Justification
			right=None, left=None, center=None,
			# Valigned too
			centered=None):
	""" 
	A simple text processing helper function.  
	Supports multi-line text via newline chars.
	"""
	if not pygame.font:
		print "Oh shiiiatt no font."
		return 
	lines = caption.split('\n')
	last_textpos = None
	for line in lines:
		font = pygame.font.Font(None, size)
		text = font.render(line, 1, color)
		textpos = text.get_rect()
		if right:
			textpos.topright = right
			move_d = (-margin, margin)
		elif left:
			textpos.topleft = left
			move_d = (margin, margin)
		elif center:
			textpos.midtop = center
			move_d = (0, margin)
		elif centered:
			textpos.center = centered
			move_d = (0, 0)
		else:
			textpos.center = surface.get_rect().center
			move_d = (0, 0)
		
		textpos.move_ip(move_d)
		if last_textpos:
			textpos.move_ip(0, last_textpos.height + last_textpos.top)
		
		surface.blit(text, textpos)
		last_textpos = textpos.copy()


##################################################################
# The TextRect thingie                                          ##
##################################################################
class TextRectException:
    def __init__(self, message = None):
        self.message = message
    def __str__(self):
        return self.message

def render_textrect(string, font, rect, text_color, justification=0, surface=None):
    """Returns a surface containing the passed text string, reformatted
    to fit within the given rect, word-wrapping as necessary. The text
    will be anti-aliased.

    Takes the following arguments:

    string - the text you wish to render. \n begins a new line.
    font - a Font object
    rect - a rectstyle giving the size of the surface requested.
    text_color - a three-byte tuple of the rgb value of the
                 text color. ex (0, 0, 0) = BLACK
    justification - 0 (default) left-justified
                    1 horizontally centered
                    2 right-justified

    Returns the following values:

    Success - a surface object with the text rendered onto it.
    Failure - raises a TextRectException if the text won't fit onto the surface.
    """

    import pygame
    
    final_lines = []

    requested_lines = string.splitlines()

    # Create a series of lines that will fit on the provided
    # rectangle.

    for requested_line in requested_lines:
        if font.size(requested_line)[0] > rect.width:
            words = requested_line.split(' ')
            # if any of our words are too long to fit, return.
            for word in words:
                if font.size(word)[0] >= rect.width:
                    raise TextRectException, "The word " + word + " is too long to fit in the rect passed."
            # Start a new line
            accumulated_line = ""
            for word in words:
                test_line = accumulated_line + word + " "
                # Build the line while the words fit.    
                if font.size(test_line)[0] < rect.width:
                    accumulated_line = test_line 
                else: 
                    final_lines.append(accumulated_line) 
                    accumulated_line = word + " " 
            final_lines.append(accumulated_line)
        else: 
            final_lines.append(requested_line) 

    # Let's try to write the text out on the surface.

    if not surface:
        surface = pygame.Surface(rect.size, SRCALPHA) 

    accumulated_height = 0 
    for line in final_lines: 
        if accumulated_height + font.size(line)[1] >= rect.height:
            raise TextRectException, "Once word-wrapped, the text string was too tall to fit in the rect."
        if line != "":
            tempsurface = font.render(line, 1, text_color)
            if justification == 0:
                surface.blit(tempsurface, (0, accumulated_height))
            elif justification == 1:
                surface.blit(tempsurface, ((rect.width - tempsurface.get_width()) / 2, accumulated_height))
            elif justification == 2:
                surface.blit(tempsurface, (rect.width - tempsurface.get_width(), accumulated_height))
            else:
                raise TextRectException, "Invalid justification argument: " + str(justification)
        accumulated_height += font.size(line)[1]

    return surface


