#!/usr/bin/env python
#    Garden Crisis, a game by Michael Bethencourt.
#   
#    (C) 2011, released under the GPL 3.0
#   
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pygame
from camplay.camplay import *
from camplay.ui import *
from camplay.fx import *
from camplay.imagecache import *
from pygame.color import THECOLORS
import pygame.camera
import math, sys, random
from pygame.locals import *
from camplay.fx import *
from camplay import fx2


__author__ = "Michael Bethencourt"
__copyright__ = "Copyright 2011, Michael Bethencourt"
__license__ = "GPL 3.0"
__email__ = "michaelpb@gmail.com"
__version__ = "0.1"


WIDTH = 640 
HEIGHT = 480 

class Item(pygame.sprite.Sprite):
	PATH = 'item_%s.png'
	ACCELERATION = 0.1
	ITEM_MARGINS = 40
	sounds = []
	def __init__(self, item_type, spin=None, x_pos=None, velocity=0.0):
		self.item_type = item_type
		self.orig_image, self.rect = load_image(Item.PATH%item_type)
		self.velo = velocity
		self.angle = 0.0
		if not Item.sounds:
			Item.sounds = [fx2.SuperSound('fx_pipe%i.ogg' % 
							i) for i in xrange(3)]
		if not hasattr(self, 'sound'):
			self.sound = Item.sounds[random.randint(0,2)]
		if not x_pos:
			left = self.ITEM_MARGINS
			right = WIDTH - self.ITEM_MARGINS
			x_pos = random.randint(left, right)
		if not spin:
			spin = random.randint(-5, 5)
		self.angular_velocity = spin
		self.rect.move_ip((x_pos, 0))
		pygame.sprite.Sprite.__init__(self)
		
	def update(self):
		# Spin
		if self.angular_velocity > 0:
			self.angle += self.angular_velocity
			self.image = pygame.transform.rotate(self.orig_image, \
						self.angle)
		else:
			self.image = self.orig_image
		self.velo += Item.ACCELERATION
		self.rect.move_ip(Game.state.wind_direction, int(self.velo))
		if self.rect.y > HEIGHT:
			self.kill()

	def get_captured_icon(self):
		return False
	
	def activate(self, player, basket):
		ABSTRACT, BITCHES
	
class Fruit(Item):
	def __init__(self, fruit_type, score, **k):
		self.score = score
		self.fruit_type = fruit_type
		if 'velocity' not in k:
			k['velocity'] = max(0, random.randint(-4,2))
		Item.__init__(self, fruit_type, **k)

	def activate(self, player, basket):
		player.score += self.score
		if self.score > 0:
			# Is good fruit
			player.stats['Fruit'] = player.stats.get('Fruit', 0)+1
		else:
			# Is stones
			player.stats['Rocks'] = player.stats.get('Rocks', 0)+1
		#print player.score
	
	def get_captured_icon(self):
		text = str(self.score)
		if self.score > 0:
			color=(0,0,0)
			text = '+' + text
		else:
			color=(255,0,0)
		surf = pygame.Surface(self.rect.size)
		surf.set_alpha(127)
		surf.fill((255,255,255))
		blit_text(surf, text, centered=surf.get_rect().center, size=32, 
					margin=0, color=color)
		
		sprite = pygame.sprite.Sprite()
		sprite.image = surf
		rect = surf.get_rect()
		rect.center = self.rect.center
		sprite.rect = rect
		return sprite

class Spell(Item): pass
class BadThing(Fruit):
	sound = None
	def __init__(self, *a, **k):
		if not BadThing.sound:
			BadThing.sound = fx2.SuperSound('fx_thud.ogg')
		Fruit.__init__(self, *a, **k)
		self.sound = BadThing.sound


class Potion(Spell):
	EXPIRATION = 500
	sound = None
	def __init__(self):
		Item.__init__(self, 'potion', spin=0)
		if not Potion.sound:
			Potion.sound = fx2.SuperSound('fx_potion.ogg')
		self.sound = Potion.sound

	def activate(self, player, basket):
		player.add_buff(Potion, self.orig_image.copy(), \
						self.EXPIRATION, 1)

class Extender(Spell):
	EXPIRATION = 1000
	sound = None
	def __init__(self):
		Item.__init__(self, 'powerup0', spin=0)
		if not Extender.sound:
			Extender.sound = fx2.SuperSound('fx_extender.ogg')
		self.sound = Extender.sound


	def activate(self, player, basket):
		player.add_buff(Extender, self.orig_image.copy(), 
				self.EXPIRATION, Basket.BASKET_STAGES)

class Wind(Spell):
	WIND_TICKS = 300
	sound = None
	def __init__(self):
		if Game.state.player_count == CamPlay.SINGLE:
			self.blow_left = random.randint(0,1)==0
			spin =  3 if self.blow_left else -3
		else:
			spin = 0
		Item.__init__(self, 'wind', spin=spin)
		if not Wind.sound:
			Wind.sound = fx2.SuperSound('fx_wind.ogg')
		self.sound = Wind.sound

	def activate(self, player, basket):
		if Game.state.player_count == CamPlay.DOUBLE:
			self.blow_left = player.team==CamPlay.LEFT
		wind_blow = -1 if self.blow_left else 1
		Game.state.winds.append((self.WIND_TICKS, wind_blow))
	
	def get_captured_icon(self):
		sprite = pygame.sprite.Sprite()
		sprite.image = self.orig_image.copy()
		sprite.rect = self.rect.copy()
		text = '<' if self.blow_left else '>'
		blit_text(sprite.image, text, color=(0,0,255), 
			centered=sprite.image.get_rect().center, size=64) 
		return sprite

class MagicHat(Spell):
	sound = None
	def __init__(self):
		Item.__init__(self, 'magichat', spin=0)
		if not MagicHat.sound:
			MagicHat.sound = fx2.SuperSound('fx_magic.ogg')
		self.sound = MagicHat.sound
	
	def activate(self, player, basket):
		def hflip(screen):
			screen.blit(pygame.transform.flip(screen, True, False), (0,0))
		
		def vflip(screen):
			screen.blit(pygame.transform.flip(screen, False, True), (0,0))

		def sflip1(screen):
			flip = pygame.transform.flip(screen, False, True)
			size = screen.get_size()
			rect = pygame.Rect((0,size[1]/2), (size[0], size[1]/2))
			screen.blit(flip, (0,0), rect)
		
		def sflip2(screen):
			flip = pygame.transform.flip(screen, True, False)
			size = screen.get_size()
			rect = pygame.Rect((size[0]/2,0), (size[0]/2, size[1]))
			screen.blit(flip, (0,0), rect)
		
		def sflip3(screen):
			size = screen.get_size()
			flip = pygame.transform.flip(screen, True, False)
			rect = pygame.Rect((0,0), (size[0]/2, size[1]))
			screen.blit(flip, (size[0]/2, 0), rect)
		
		roll = random.randint(0, 100)
		TIME = 500
		if roll < 20:
			Game.state.add_graphic_effect(hflip, TIME)
		elif roll < 40:
			Game.state.add_graphic_effect(vflip, TIME)
		elif roll < 60:
			Game.state.add_graphic_effect(sflip1, TIME)
		elif roll < 80:
			Game.state.add_graphic_effect(sflip2, TIME)
		else:
			Game.state.add_graphic_effect(sflip3, TIME)



class Basket(Sprite):
	# imagecache.Sprite
	PATH = 'player_%s.png'
	BASKET_STAGES = 3
	def __init__(self, player):
		self.player = player
		color = {
			CamPlay.SINGLE: "single", 
			CamPlay.LEFT: "green", 
			CamPlay.RIGHT: "red2", 
		}[player.team]
		Sprite.__init__(self, Basket.PATH%color, (0.1, 0.1))
		self.stats = {}
	
	@property
	def basket_size(self):
		return min(self.BASKET_STAGES, \
			len(self.player.get_buffs(Extender)))

	def update(self):
		self.prect.center = PRect.from_rect(pygame.Rect(
				self.player.get_center2d(), (0, 0))).center
		new_scale = (self.basket_size)*0.04 + 0.12
		self.scale_ip((new_scale, new_scale))
		#print "NEW SCALE", new_scale
		if self.player.is_touching_other_player:
			a = 0.52 if self.player.team==CamPlay.RIGHT else -0.52
			self.rotate_ip(a)
		else: 
			self.rotate_ip(0.0)
			
		if self.player.get_buffs(Potion):
			mask = self.player.get_motion_mask()
			rects = mask.get_bounding_rects()
			MIN_AREA = 300
			def supercollider(left, right): 
				if pygame.sprite.collide_rect(left, right):
					return True
				for rect in rects:
					w,h = rect.size
					if w*h > MIN_AREA:
						a = right.rect.colliderect(rect)
						if a: return True
				return False
			items = pygame.sprite.spritecollide(self, 
					Game.state.items, True, supercollider)
		else:	
			items = pygame.sprite.spritecollide(self, 
					Game.state.items, True)
		
		for item in items:
			item.activate(self.player, self)
			item.sound.play()
			sprite.fly_into(
				Game.state.anim_under_basket, item, self, 10)
			ci = item.get_captured_icon()
			if ci:
				sprite.float_up_and_vanish(
					Game.state.anim_over_basket, ci, 20)
			


class GameState(CachedMutable):
	def __init__(self):
		self.items = pygame.sprite.Group()
		self.baskets = pygame.sprite.Group()
		self.anim_over_basket = pygame.sprite.Group()
		self.anim_under_basket = pygame.sprite.Group()
		self.winds = []
		self.players = None
		self.fx = []
		CachedMutable.__init__(self)
	
	@mutable	
	def update(self):
		self.items.update()
		self.anim_under_basket.update()
		self.baskets.update()
		self.anim_over_basket.update()
		# tick everything down
		self.winds = map(lambda w: (w[0]-1, w[1]), self.winds)
		self.winds = filter(lambda w: w[0]>0, self.winds)
		#if not self.winds: # Wind is empty, turn off wind sound effect
		
		
	def add_graphic_effect(self, effect, timeout):
		self.fx.append((effect, timeout))

	def apply_graphic_effects(self, screen):
		for player in self.players:
			if player.get_buffs(Potion):
				# Apply lightning effect
				mask = player.get_motion_mask()
				points = mask.outline(50)
				if len(points)>10:
					pygame.draw.polygon(screen, (255, 255,
						255), points, 1)
				points = mask.outline(100)
				if len(points)>10:
					pygame.draw.polygon(screen, (127, 127,
						255), points, 2)
	
		self.fx = [(e, t - 1) for e, t in self.fx if t > 0]
		for effect, timeout in self.fx:
			effect(screen)
	
	@property
	@cached
	def wind_direction(self):
		return sum(map(lambda w: w[1], self.winds))


class Game: 
	state = None
	
class Stage(object):
	def __init__(self, number, quantities, rate=80, wave_size=5, velo=0):
		fp = 'stage_%i.png'%number
		self.image, self.rect = load_image(fp)
		self.image = pygame.transform.scale(self.image, (331, 192))
		self.rect = self.image.get_rect()
		self.items = []
		self.rate = rate
		self.velo = velo
		self.wave_size = wave_size
		self.quantities = quantities
		self.delay = 100 # 100 ticks upon starting
		self.reset()

	def reset(self):
		self.tick = 1
		for constructor, quantity in self.quantities.iteritems():
			for i in xrange(quantity):
				item = constructor()
				self.items.append(item)
				if self.velo:
					item.velo += random.randint(0, self.velo)
		# get a random ordering for hte items
		random.shuffle(self.items)
	
	def update(self, game_state, tela):
		self.tick += 1
		if self.tick == self.delay: # Do the start thing once
			self.start(game_state, tela)
		if self.tick < self.delay*3:
			return True
		if self.tick % self.rate == 0:
			lst = self.items[:self.wave_size] if len(self.items) > \
						self.wave_size else self.items
			for item in lst:
				game_state.items.add(item)
			self.items = self.items[len(lst):]
		return bool(self.items)

	def start(self, game_state, screen):
		period = int(self.delay / 3)
		s_rect = screen.get_rect()
		self.rect.center = s_rect.center
		start = (s_rect.midtop[0] - self.rect.width/2, s_rect.midtop[1])
		end = (s_rect.center[0] - self.rect.width/2, s_rect.center[1] - self.rect.height/2)
		anim = Animation(self.image, self.rect,
			effects.Glide(period=period, start=start, end=end,
				easing=easing.fast, blocking=True),
			effects.Grow(period=period, start_size=1.0,
				end_size=1.1, blocking=True),
			effects.Glide(period=period, end=start, start=end,
				easing=easing.fast, blocking=True),
			effects.Kill())
		game_state.anim_over_basket.add(anim)


def main():
	if not pygame.font: print 'Warning, fonts disabled'
	if not pygame.mixer: print 'Warning, sound disabled'
	pygame.init()
	pygame.init()
	size = (WIDTH, HEIGHT)
	screen = pygame.display.set_mode(size, 0)
	pygame.display.set_caption('Garden Crisis')
	clock = pygame.time.Clock()
	import camplay.imagecache
	camplay.imagecache.reset(resolution=(640, 480), datafolder='data')
	camplay.imagecache.reset_sound(datafolder='data')
	camplay = CamPlay(screen, size, debug=False, clock=clock)
	def make_menu_screen():
		# Load logo, and position on screen
		logo, logo_rect = load_image('logo_2.png')
		screen_rect = screen.get_rect()
		new_height = min(screen_rect.height - 
				screen_rect.centery, logo_rect.height)
		# Scale logo to new height
		#logo_rect.width *=  new_height / logo_rect.height
		#logo_rect.height = new_height
		#logo = pygame.transform.scale(logo, logo_rect.size)
		#logo_rect.center = screen_rect.center
		logo_rect.midtop = screen_rect.center
		logo_rect.move_ip(0, -100)
		anim = Animation(logo, logo_rect, comboeffects.FlipInAndThrob(200))
		anims = pygame.sprite.Group(anim)
		return anims
	
	def make_pause_screen():
		logo, logo_rect = load_image('logo_paused.png')
		logo_rect.center = screen.get_rect().center
		sprite = pygame.sprite.Sprite()
		sprite.image = logo
		sprite.rect = logo_rect
		return  pygame.sprite.Group(sprite)

	camplay.modal.theme(
		menu_sprites=make_menu_screen(), 
		pause_sprites=make_pause_screen())
	main_menu(camplay, clock, screen)



def main_menu(camplay, clock, screen):
	ABOUT = 'a CamPlay game by michaelb\n'+\
		'http://michaelb.org/camplay/\n' +\
		'released under the GPL 3.0\n' + \
		'see website for artwork credits'
	DEMO = "Let people just walk up to play.\n"+\
		"It's like a fun, interactive art display.\n"+\
		"(To quit, hit ESC or close the window)"
	# Once I get better text elements, this will go away.
	labels = lambda txt: map(Label, txt.split('\n'))
	# Add license for graphics here
	quit = False
	while not quit:
		player_choice = camplay.modal.menu(['One player', 'Versus', 
								'Help'])
		if isinstance(player_choice, str) and 'elp' in player_choice:
			image = ImageWidget('help.png', 
						css={'margin': Margin(10)})
			#choice = camplay.modal.dialog(image,
			#	buttons=("Back", "About", "Demo mode"), 
			#	use_spacing=False)
			choice = camplay.modal.dialog(image,
				buttons=("Back", "About"), 
				use_spacing=False)
			if "about" in choice:
				c = AutoGridContainer(*labels(ABOUT),columns=1)
				camplay.modal.dialog(c, buttons=("Okay",))
			#elif "demo" in choice:
			#	c = AutoGridContainer(*labels(DEMO),columns=1)
			#	c = camplay.modal.dialog(c, 
			#		buttons=("Cancel", "Start demo"))
			#	if 'demo' in c:
			#		start_demo_mode(camplay, clock, screen)

		else:
			if type(player_choice) is bool:
				player_choice = 'oneplayer'
			stageset_choice = camplay.modal.menu(['Quick',
						'Standard', 'Challenge'])
			if type(stageset_choice) is bool:
				stageset_choice = 'quick'
			start_game(player_choice, camplay, clock, 
						screen, stageset_choice)


def start_game(choice, camplay, clock, screen, stageset_choice, 
						already_calibrated=False):
	game_state = GameState()
	Game.state=game_state
	frame, frame_rect = load_image('frame2.png')
	frame = pygame.transform.scale(frame, screen.get_size())
	if not choice:
		choice = 'one'
	if 'one' in choice:
		# One player
		camplay.new_game(CamPlay.SINGLE)
		if not already_calibrated:
			camplay.modal.calibrate_sequence()
		game_state.baskets.add(Basket(camplay.players[0]))
		game_state.player_count = CamPlay.SINGLE
		game_state.players = camplay.players
	elif 'vers' in choice:
		# Two player
		camplay.new_game(CamPlay.DOUBLE)
		if not already_calibrated:
			camplay.modal.calibrate_sequence()
		game_state.baskets.add(Basket(camplay.players[0]))
		game_state.baskets.add(Basket(camplay.players[1]))
		game_state.player_count = CamPlay.DOUBLE
		game_state.players = camplay.players
	
	# PICK THE STAGE STAGES
	stage_sets = get_stages()
	stage_set = None
	for name, stages in stage_sets:
		if name.lower().replace(' ', '') in \
				stageset_choice.lower().replace(' ',''):
			loaded_stages = stages
			break
	
	def check_events(events):
		for event in events:
			if event.type == QUIT:
				camplay.stop()
				sys.exit(0)
				return
			elif event.type == KEYDOWN and event.key == K_ESCAPE:
				camplay.stop()
				sys.exit(0)
				return
			elif event.type == KEYDOWN:
				if event.key == K_SPACE:
					camplay.calibrate()
				if event.key == K_f:
					pygame.display.toggle_fullscreen()
	############### ############### ############### ###############
	# Set up music
	pygame.mixer.music.set_volume(0.5)
	playlist = fx2.PlayList('music_2_the_countryside_wakes_up.ogg')
	playlist.start()
	############### ############### ############### ###############
	
	while loaded_stages:
		stage = loaded_stages.pop(0)
		while stage.update(game_state, screen):
			check_events(pygame.event.get())

			fx2.update()
			
			game_state.update()
			camplay.tick(True)
			game_state.items.draw(screen)
			game_state.anim_under_basket.draw(screen)
			game_state.baskets.draw(screen)
			game_state.anim_over_basket.draw(screen)
				
			# The orchard themed frame
			screen.blit(frame, (0,0))

			# Do the crazy effects
			game_state.apply_graphic_effects(screen)
			
			
			#camplay.blit_ui()
			camplay.blit_hud()
			#overlay.oldtv(screen, color='G')

			pygame.display.flip()

			# Keep framerate!
			clock.tick(60)

	# SCORE time! :)
	replay_choice = camplay.modal.score_sequence(("Play again!", 
				"Main menu"))
	if "again" in replay_choice:
		start_game(choice, camplay, clock, screen,
				stageset_choice, True)
	playlist.stop()
	# Otherwise, it just goes back to the main menu


def start_demo_mode(camplay, clock, screen):
	"""
	Demo mode doesn't have everything the other one has,
	so we skip a few things, like graphic effects, and the
	HUD.
	"""
	while True:
		check_events(pygame.event.get())
		camplay.tick(True)
		game_state.items.draw(screen)
		game_state.anim_under_basket.draw(screen)
		game_state.baskets.draw(screen)
		game_state.anim_over_basket.draw(screen)
			
		# The orchard themed frame
		screen.blit(frame, (0,0))
		
		pygame.display.flip()
		clock.tick(60)


# Some simple constructor methods:
def redapple():
	return Fruit('redapple', 5)
def greenapple():
	return Fruit('greenapple', 15)
def yellowapple():
	x_pos = 0
	# FIXME when resolution becomes unfixed
	if len(Game.state.players) > 1 and \
				random.randint(0, 2)!=2:
		x_pos += random.randint(-10, 10) + WIDTH/2
	return Fruit('yellowapple', 50, x_pos=x_pos)

def _get_velo():
	return {'velocity': random.randint(0, 8), 
		'spin': random.randint(-10, 10)}
def rock0():
	k = _get_velo()
	return BadThing('rock0', -5, **k)
def rock1():
	k = _get_velo()
	return BadThing('rock1', -10, **k)
def rock2():
	k = _get_velo()
	return BadThing('rock2', -30, **k)

def get_stages():
	return [
		('Quick', [
			Stage(1, {
				redapple: 80,
			}),
			Stage(2, { redapple: 50, greenapple: 25, yellowapple: 3,
			}),
			Stage(11, { redapple: 10, greenapple: 10, yellowapple: 5,
				rock0: 25, rock1: 10, rock2: 5,
			})]),
		('Standard', [
			Stage(1, {
				redapple: 80,
			}),
			Stage(2, { redapple: 50, greenapple: 25, yellowapple: 3,
			}),
			Stage(3, { redapple: 10, greenapple: 10, yellowapple: 5,
				rock0: 25, rock1: 10, rock2: 5,
			}),
			Stage(4, { Extender: 2, redapple: 25, greenapple: 10,
				rock0: 5, rock1: 5, rock2: 5,
			}),
			Stage(5, { Extender: 2, Wind: 10,
				redapple: 50, greenapple: 10, yellowapple: 2,
				rock0: 5, rock1: 5,
			}),
			Stage(6, { Extender: 2, Wind: 40,
				redapple: 200, greenapple: 100,
				rock0: 50,
			}, wave_size=15),
			Stage(7, { rock1: 50, rock2: 50,
				MagicHat: 1, yellowapple: 10,
			}, rate=40, velo=4),
			Stage(8, { Wind: 10, Potion: 1, Extender: 2,
				redapple: 50, greenapple: 10, yellowapple: 2,
				rock0: 5, rock1: 5,
			}),
			Stage(9, {
				Wind: 40, Potion: 2, MagicHat: 40, Extender: 10,
				redapple: 50, greenapple: 10, yellowapple: 5,
				rock0: 10, rock1: 10, rock2: 10,
			}),
			Stage(10, {
				Wind: 30, Potion: 2, MagicHat: 10, Extender: 2,
				redapple: 50, greenapple: 10, yellowapple: 5,
				rock0: 10, rock1: 10, rock2: 10,
			}, rate=40, velo=10),
			Stage(11, {
				rock0: 400, Potion: 2, rock1: 50, rock2: 50,
				yellowapple: 100, MagicHat: 10,
			}, rate=20, wave_size=20, velo=20),
		]),
		('Challenge', [
			Stage(1, {
				redapple: 80,
			}, rate=15, wave_size=2, velo=10),
			Stage(2, { redapple: 50, greenapple: 25, yellowapple: 3,
			}, rate=15, wave_size=50, velo=30),
			Stage(3, { redapple: 1, greenapple: 10, yellowapple: 5,
				rock0: 25, rock1: 10, rock2: 5,
			}, rate=15, wave_size=2, velo=20),
			Stage(4, { Extender: 2, redapple: 25, greenapple: 10,
				rock0: 5, rock1: 5, rock2: 5,
			}, rate=15, wave_size=2, velo=20),
			Stage(5, { Extender: 2, Wind: 10,
				redapple: 50, greenapple: 10, yellowapple: 2,
				rock0: 5, rock1: 5,
			}, rate=15, wave_size=2, velo=20),
			Stage(6, { Extender: 2, Wind: 40,
				redapple: 200, greenapple: 100,
				rock0: 50,
			}, rate=15, wave_size=4, velo=20),
			Stage(7, { rock1: 50, rock2: 50,
				MagicHat: 1, yellowapple: 10,
			}, rate=7, wave_size=2, velo=15),
			Stage(8, { Wind: 10, Potion: 1, Extender: 2,
				redapple: 50, greenapple: 10, yellowapple: 2,
				rock0: 5, rock1: 5,
			}, rate=15, wave_size=2, velo=20),
			Stage(9, {
				Wind: 40, Potion: 1, MagicHat: 40, Extender: 10,
				redapple: 50, greenapple: 10, yellowapple: 5,
				rock0: 10, rock1: 10, rock2: 10,
			}, rate=15, wave_size=2, velo=20),
			Stage(10, {
				Wind: 30, Potion: 1, MagicHat: 10, Extender: 2,
				redapple: 50, greenapple: 10, yellowapple: 5,
				rock0: 10, rock1: 10, rock2: 10,
			}, rate=7, wave_size=2, velo=40),
			Stage(11, {
				rock0: 400, Potion: 1, rock1: 50, rock2: 50,
				yellowapple: 100, MagicHat: 10,
			}, rate=7, wave_size=5, velo=60), 
		]),
	]





if __name__=='__main__':
	main()


